/*		C O M _ P O R T S .  C					*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>	/* Unix standard function defs			*/
#include <fcntl.h>	/* file control defs				*/
#include <errno.h>
#include <termios.h>	/* POSIX terminal control defs			*/
#include "com_ports.h"	/* com port stuf				*/


#ifndef ESC
  #define ESC 27
#endif

#ifndef TRUE
  #define TRUE 1
  #define FALSE 0
#endif

static struct termios old_options;
static struct termios options;
static int configured = FALSE;
typedef struct rate
{
  char *rate;			/* baud character string		*/
  speed_t speed;		/* corresponding rate			*/
} RATE;

static RATE rate_table[] =
{
  { "38400", B38400 },
  { "19200", B19200 },
  { "9600", B9600 },
  { "4800", B4800 },
  { "2400",B2400 },
  { "1200", B1200 },
  { "600", B600 },
  { "300", B300 },
  { "200", B200 },
  { "150", B150 },
  { "134", B134 },
  { "134.5", B134 },
  { "110", B110 },
  { "75", B75 },
  { "50", B50 }
};				/* end of rate table			*/

/*START******************************************************************
*	TITLE:		open_port - open a com port
*/
int open_port( int port )
{
/*	DESCRIPTION:	
*
**END********************************************************************/
int port_fd;					/* file descripter
*/

if( port == COM1 )		
  port_fd = open( "/dev/ttyS0", O_RDWR | O_NOCTTY | O_NDELAY );
else if( port == COM2 )
  port_fd = open( "/dev/ttyS1", O_RDWR | O_NOCTTY | O_NDELAY );
else
{
  return(-1);
}
return( port_fd );
}
/****	END OF FUNCTION		open_port()				*/

/*START******************************************************************
*	TITLE:		close_port - close a com port
*/
void close_port( int port_fd )
{
/*	DESCRIPTION:	
*
**END********************************************************************/

if( configured )
{
  tcsetattr(port_fd, TCSANOW, &old_options);  /* restore changes	*/
  configured = FALSE;
}
close(port_fd);
}
/****	END OF FUNCTION		close_port()				*/

/*START******************************************************************
*	TITLE:		port_putc - put a character to a port
*/
int port_putc( int port_fd, char chr )
{
/*	DESCRIPTION:	
*
**END********************************************************************/
int rtn;

rtn = write(port_fd, &chr, sizeof(char) );
return( rtn );
}
/****	END OF FUNCTION		port_putc()				*/

/*START******************************************************************
*	TITLE:		port_getc - put a character to a port
*/
int port_getc( int port_fd )
{
/*	DESCRIPTION:	gets a character from the port and returns it
*
**END********************************************************************/
int rtn;
int chr = 0;

rtn = read(port_fd, &chr, sizeof(char) );
if( rtn < 0 )
  return( rtn );
return( chr);
}
/****	END OF FUNCTION		port_getc()				*/

/*START******************************************************************
*	TITLE:		port_putmem - put x bytes of memory to port
*/
int port_putmem( int port_fd, char *mem, int count )
{
/*	DESCRIPTION:	
*
**END********************************************************************/

int cnt;
char *cp;
                                        
for( cnt = 0, cp = mem; cnt != count; ++cnt )
  port_putc( port_fd, *cp++ );
}
/****	END OF FUNCTION		port_putmem()				*/

/*START******************************************************************
*	TITLE:		port_putcrlf - put CR, LF
*/
int port_putcrlf( int port_fd )
{
/*	DESCRIPTION:	
*
**END********************************************************************/

port_putc( port_fd, 0x0d );			      
port_putc( port_fd, 0x0a );			      
}
/****	END OF FUNCTION		port_putcrlf				*/

/*START******************************************************************
*	TITLE:		port_configure - configure port	
*/
int port_configure(int port_fd, char *mode )
{
/*	DESCRIPTION:	get port_fd first by doing an open_port().
*		call example - port_configure( port_fd, "9600,n,8,1");
*		returns -1 if parameters are bogus.
*
**END********************************************************************/
#define BUF 1024
#define SAME 0
#define R(x) (rptr->x)
char baud[BUF];
char parity[BUF];
char *cp, *cp1;
RATE *rptr;

/* elements in rate_table */
int elements = ( sizeof(rate_table) ) /
		 ( sizeof(char *) + sizeof(speed_t) );

/* extract baud and parity from mode input		*/
for(cp = mode, cp1 = baud; *cp != ',' && *cp != '\0'; )
  *cp1++ = *cp++;
for(++cp, cp1 = parity; *cp != '\0'; )
  *cp1++ = *cp++;

fcntl(port_fd, F_SETFL, O_NONBLOCK );   /* set for non-blocking I/O 	*/
tcgetattr(port_fd, &old_options );      /* escrow current settings	*/
options = old_options;
configured = TRUE;			/* for close_port() use 	*/

/* pull the baud rate from the table					*/
for( rptr = rate_table; elements; ++rptr, elements-- )
{
  if( strcmp( R(rate), baud ) == SAME )
  {
    cfsetispeed( &options, R(speed) );
    cfsetospeed( &options, R(speed) );
    break;
  }
}

if( strcmp( parity, "8n1" ) == SAME )
{
  options.c_cflag |= (CLOCAL | CREAD );
  options.c_cflag &= ~PARENB;             /* no parity*/
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS8;
  options.c_cflag &= ~CRTSCTS;       /* no hardware flow control*/
  options.c_lflag &= ~(ICANON | ECHO | ISIG);     /* raw input */
}
else if( strcmp( parity, "7e1" ) == SAME )
{
  options.c_cflag |= (CLOCAL | CREAD );
  options.c_cflag |= PARENB;             /* even  parity*/
  options.c_cflag &= ~PARODD;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS7;
}
else if( strcmp( parity, "7o1" ) == SAME )
{
  options.c_cflag |= (CLOCAL | CREAD );
  options.c_cflag |= PARENB;             /* even  parity*/
  options.c_cflag |= PARODD;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS7;
}

options.c_cflag &= ~CRTSCTS;       /* no hardware flow control*/
options.c_lflag &= ~(ICANON | ECHO | ISIG);     /* raw input */
tcsetattr(port_fd, TCSANOW, &options);  /* make the changes now         */
return(0);
}
/****	END OF FUNCTION		port_configure()			*/

